
// MpegReceiver.cpp : MPEG program stream receiver classes definitions
// Copyright (C) 2000-2005 Canopus Co., Ltd. All rights reserved.

#include "stdafx.h"
#include "MpegReceiver.h"

#ifdef __TRACEOUT
BOOL g_bLogOut = FALSE;
UINT g_nTimerID = 0;
UINT g_nDecID = 0;
VOID CALLBACK LogOutTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);
#endif
/////////////////////////////////////////////////////////////////////////////
// CMpegReceiver

IMPLEMENT_DYNAMIC(CMpegReceiver, CMpegTransfer)

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiver constructor / destructor

CMpegReceiver::CMpegReceiver(UINT nDecId)
	: CMpegTransfer(nDecId),
	m_hEventStop(WSA_INVALID_EVENT), m_pReceiveThread(NULL),
	m_fRestart(FALSE), m_fStopReq(FALSE), m_nDiff(0), m_fCheckStc(FALSE)

{
	RegisterReceiver(m_nId, this);
}

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiver static members

CCriticalSection CMpegReceiver::m_csec; // for exclusive access to m_listDecCallbacks
CTypedPtrList<CPtrList, CDecCallback*> CMpegReceiver::m_listDecCallbacks;
CCriticalSection CMpegReceiver::m_csecThread; // for exclusive access to m_pReceiverThread
CMpegReceiverThread* CMpegReceiver::m_pReceiverThread = NULL;
CMpegReceiverWnd* CMpegReceiver::m_pReceiverWnd = NULL;

CMpegReceiver::~CMpegReceiver()
{
	ReceiveStop();
	DiscardBuffs();
	Disconnect();
	RegisterReceiver(m_nId, NULL);
}

BOOL CMpegReceiver::Init(UINT nDecId, CMpegReceiver* pReceiver /* = NULL */,
	DEC_CB_STATUS pfnStatus /* = NULL */, DEC_CB_ERROR pfnError /* = NULL */)
{
	BOOL fResult = FALSE;
	m_csecThread.Lock();
	if (!m_pReceiverThread && (m_pReceiverThread = (CMpegReceiverThread*)AfxBeginThread(RUNTIME_CLASS(CMpegReceiverThread),
		THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED))) { // one time initialization
		m_pReceiverThread->m_bAutoDelete = FALSE; m_pReceiverThread->m_pThreadParams = &m_pReceiverWnd;
		VERIFY(m_pReceiverThread->ResumeThread() == 1);
	}
	m_csecThread.Unlock();
	if (!m_pReceiverThread)
		goto done;
	m_csec.Lock();
	m_listDecCallbacks.AddTail(new CDecCallback(nDecId, NULL, pfnStatus, pfnError));
	m_csec.Unlock();
	if (DEC_Set_Callback(nDecId, DecStatusCallback, DecErrorCallback) != DEC_SUCCESS)
		goto done;
	DEC_BSR_PARAMETER Param;
	Param.fType = DEC_BSR_TYPE_PS;
	Param.pfnCallback = BsrCallback;
	Param.cbBuff = m_cbBuffDefault; Param.cBuff = m_cBuffDefault; Param.dwData = (DWORD)pReceiver;
	if (DEC_Initialize_Ex(nDecId, DEC_MEDIA_MEMORY, &Param) != DEC_SUCCESS)
		goto done;

	DEC_DECODE_PARAMETER dec_param;
	if (DEC_Get_Decode_Parameter(nDecId, &dec_param) != DEC_SUCCESS)
		goto done;

	dec_param.fRepeatOnStop = TRUE;
	dec_param.fFrameRepeat = FALSE;

	if (DEC_Set_Decode_Parameter(nDecId, &dec_param) != DEC_SUCCESS)
		goto done;
	fResult = TRUE;
done:
	if (!fResult)
		Exit(nDecId);
	return fResult;
}

BOOL CMpegReceiver::Exit(UINT nDecId)
{
	VERIFY(DEC_Terminate(nDecId) == DEC_SUCCESS);
	m_csec.Lock();
	POSITION pos, posPrev;
	for (pos = m_listDecCallbacks.GetHeadPosition(); posPrev = pos; ) {
		CDecCallback* pCallback = m_listDecCallbacks.GetNext(pos);
		if (pCallback->m_nDecId == nDecId) {
			m_listDecCallbacks.RemoveAt(posPrev); delete pCallback;
			break;
		}
	}
	m_csec.Unlock();
 	if (!posPrev)
		goto done;
	m_csecThread.Lock();
	if (m_listDecCallbacks.IsEmpty() && m_pReceiverThread) { // one time termination
		DWORD dwExitCode;
		VERIFY(GetExitCodeThread(m_pReceiverThread->m_hThread, &dwExitCode));
		if (dwExitCode == STILL_ACTIVE) {
			ASSERT(IsWindow(m_pReceiverWnd->GetSafeHwnd()));
			 // request sender thread to terminate itself, and wait for it
			VERIFY(m_pReceiverWnd->PostMessage(WM_CLOSE, 0, 0));
			WaitForSingleObject(m_pReceiverThread->m_hThread, INFINITE);
			delete m_pReceiverThread; m_pReceiverThread = NULL;
			m_pReceiverWnd = NULL;
		}
	}
	m_csecThread.Unlock();
done:
	return TRUE;
}

BOOL CMpegReceiver::RegisterReceiver(UINT nDecId, CMpegReceiver* pReceiver) // register / unregister CMpegReceiver instance
{
	m_csec.Lock();
	for (POSITION pos = m_listDecCallbacks.GetHeadPosition(); pos; ) {
		CDecCallback* pCallback = m_listDecCallbacks.GetNext(pos);
		if (pCallback->m_nDecId == nDecId) {
			pCallback->m_pReceiver = pReceiver;
			m_csec.Unlock();
			return TRUE;
		}
	}
	m_csec.Unlock();
	return FALSE;
}

CDecCallback* CMpegReceiver::FindDecCallback(UINT nDecId)
{
	m_csec.Lock();
	for (POSITION pos = m_listDecCallbacks.GetHeadPosition(); pos; ) {
		CDecCallback* pCallback = m_listDecCallbacks.GetNext(pos);
		if (pCallback->m_nDecId == nDecId) {
			m_csec.Unlock();
			return pCallback;
		}
	}
	m_csec.Unlock();
	return NULL;
}

// the following callbacks may be called in the context of the thread other than the one for theApp

void CALLBACK CMpegReceiver::DecStatusCallback(UINT nDecId, DEC_STATUS_NOTIFY fStatus)
{
	if (m_pReceiverWnd)
		m_pReceiverWnd->PostMessage(MSG_DEC_STATUS, (WPARAM)nDecId, (LPARAM)fStatus);
}

void CALLBACK CMpegReceiver::DecErrorCallback(UINT nDecId, ULONG ulError)
{
	if (m_pReceiverWnd)
		m_pReceiverWnd->PostMessage(MSG_DEC_ERROR, (WPARAM)nDecId, (LPARAM)ulError);
}

void CALLBACK CMpegReceiver::BsrCallback(LPBYTE pbBuff, DWORD cbWrite, LPDWORD pcbWritten, DWORD dwData)
{
	if (dwData) {
		CMpegReceiver* pReceiver = (CMpegReceiver*)dwData;
		if (pReceiver->BsrCallback(pbBuff, cbWrite, pcbWritten)) {
			ASSERT(m_pReceiverWnd);
			m_pReceiverWnd->PostMessage(MSG_DEC_STOP, pReceiver->m_nId, 0);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiver members

void CMpegReceiver::DiscardBuffs()
{
	try
	{
		while (!m_listRcvBuffs.IsEmpty())
			delete[] m_listRcvBuffs.RemoveHead();
		while (!m_listBsrBuffs.IsEmpty())
			delete[] m_listBsrBuffs.RemoveHead();
	}
	catch (CException* e)
	{
		e->Delete();
	}
}

BOOL CMpegReceiver::Connect(LPCTSTR pszAddress, WORD nPort, LPCTSTR pszAddressOpposite)
{
	if ((m_hEventStop = WSACreateEvent()) == WSA_INVALID_EVENT)
		return FALSE;
	return CMpegTransfer::Connect(pszAddress, nPort, pszAddressOpposite);
}

BOOL CMpegReceiver::Disconnect()
{
	BOOL fResult = CMpegTransfer::Disconnect();
	if (fResult && m_hEventStop != WSA_INVALID_EVENT) {
		VERIFY(WSACloseEvent(m_hEventStop)); m_hEventStop = WSA_INVALID_EVENT;
	}
	return fResult;
}

BOOL CMpegReceiver::ReceiveStart(DWORD cbBssBuff /* = m_cbBuffDefault */, DWORD cBssBuff /* = m_cBuffDefault */)
{
	m_eventStopped.SetEvent();

	ASSERT(m_fState == trsConnected || m_fState == trsStopped);
	DiscardBuffs();
	m_cbBuff = cbBssBuff; m_cBuff = cBssBuff;
	try { // allocate receive buffers (receive length will be stored to the 1st 4 bytes of each buffer)
		for (DWORD c = m_cBuff; c; c--)
			m_listRcvBuffs.AddTail(new BYTE[m_cbBuff + sizeof(DWORD)]);
	}
	catch (CException* e) {
		e->Delete(); goto done;
	}
#ifdef _DEBUG
	m_cRcvBuffMinimum = m_listRcvBuffs.GetCount();
#endif
	if (!(m_pReceiveThread = AfxBeginThread(ReceiveThread, this)))
		goto done;
	DEC_BSR_PARAMETER Param;
	if (DEC_Get_BSR_Parameter(m_nId, &Param) != DEC_SUCCESS)
		goto done;
	Param.cbBuff = m_cbBuff; Param.cBuff = m_cBuff; Param.dwData = (DWORD)this;
	if (DEC_Set_BSR_Parameter(m_nId, &Param) != DEC_SUCCESS)
		goto done;
	if (DEC_Play(m_nId) != DEC_SUCCESS)
		goto done;
	m_fState = trsStarting;
done:
	if (m_fState == trsStarting)
		return TRUE;
	else {
		TerminateReceiveThread();
		DiscardBuffs();
		return FALSE;
	}
}

BOOL CMpegReceiver::ReceiveStop()
{
	if (m_fState != trsStarting && m_fState != trsStarted) return TRUE;

	TerminateReceiveThread();
	m_fRestart = FALSE;
	return DecodeStop();
}

void CMpegReceiver::TerminateReceiveThread()
{
	if (m_pReceiveThread) {
		VERIFY(WSASetEvent(m_hEventStop)); // wait for receive thread to terminate itself
		WaitForSingleObject(m_pReceiveThread->m_hThread, INFINITE);
		// at this point, m_pDisonnectThread was already deleted in AfxEndThread
		m_pReceiveThread = NULL;
	}
	if (m_hEventStop != WSA_INVALID_EVENT)
		VERIFY(WSAResetEvent(m_hEventStop));
}

BOOL CMpegReceiver::DecodeStop()
{
	m_eventStop.SetEvent();

	if (m_fState == trsStopped || m_fState == trsDisconnected)
		goto done;
	if (m_fState == trsStarting) { // no data received
		LPBYTE pbBuff = GetRcvBuff();
		*(LPDWORD)pbBuff = 0;
		PutBsrBuff(pbBuff);
	}

	if (m_fState == trsStarting || m_fState == trsStopping)
	{
		DEC_Stop(m_nId);
	}
	else
	{
		m_eventStopped.Lock(); 
	}
	m_fState = trsStopped;
done:
	return (m_fState != trsStopped && m_fState != trsDisconnected) ? FALSE : TRUE;
}

void CMpegReceiver::OnDecStopped()
{
	TRACE1("min. receive buffer count = %d\n", m_cRcvBuffMinimum);
}

UINT ReceiveThread(LPVOID pParam)
{
// note: Sometimes the very 1st data does not arrive.
	CMpegReceiver* pReceiver = (CMpegReceiver*)pParam;

	ULONG uError = 0;

	pReceiver->m_eventStop.ResetEvent();
	pReceiver->m_fRestart = FALSE;
	pReceiver->m_fStopReq = FALSE;

	BOOL fBuffEmpty = FALSE;

	LPBYTE pbBuff = NULL;
	while (TRUE) {
		if (pReceiver->m_fRestart || pReceiver->m_fStopReq)
			pbBuff = NULL;
		else
			pbBuff = pReceiver->GetRcvBuff();

		//failed to get buffer or restarting decoder
		if (!pbBuff)
		{
			//use temporary buffer
			pbBuff = new BYTE[pReceiver->m_cbBuff + sizeof(DWORD)];

			DEC_STATUS dec_status;
			if (DEC_Get_Status(pReceiver->m_nId, &dec_status) == DEC_SUCCESS &&
				dec_status == DEC_STATUS_PLAY)
			{
				pReceiver->m_fRestart = TRUE;

				//stopping decoder
				pReceiver->m_eventStop.SetEvent();

				if (!fBuffEmpty)
				{
#ifdef __TRACEOUT
					TraceOut("stopping decoder");
#endif

					//clear buffer
					pReceiver->DiscardBuffs();
					try {
						for (DWORD c = pReceiver->m_cBuff; c; c--)
							pReceiver->m_listRcvBuffs.AddTail(new BYTE[pReceiver->m_cbBuff + sizeof(DWORD)]);
					}
					catch (CException* e) {
						e->Delete();
					}
					fBuffEmpty = TRUE;
				}
			}
		}

		VERIFY(WSAResetEvent(pReceiver->m_hEvent));
		WSABUF Buff;
		DWORD cbReceived = 0, dwFlags = 0;
		int cbAddr = sizeof(SOCKADDR_IN);
		WSAOVERLAPPED Overlapped;
		Buff.len = pReceiver->m_cbBuff; Buff.buf = (LPSTR)(pbBuff + sizeof(DWORD));
		Overlapped.hEvent = pReceiver->m_hEvent;
		if (!WSARecvFrom(pReceiver->m_hSocket, &Buff, 1,
			&cbReceived, &dwFlags, (LPSOCKADDR)&pReceiver->m_AddrOpposite, &cbAddr, &Overlapped, NULL))
			ASSERT(cbReceived <= pReceiver->m_cbBuff);
		else if (WSAGetLastError() == WSA_IO_PENDING) { // operation is not completed
			WSAEVENT hEvents[2];
			hEvents[0] = pReceiver->m_hEventStop; hEvents[1] = pReceiver->m_hEvent;
			switch (WSAWaitForMultipleEvents(2, hEvents, FALSE, INFINITE, FALSE)) {
			case WSA_WAIT_EVENT_0: // receive thread is required to terminate itself
				if (pReceiver->m_fRestart)
					delete[] pbBuff;
				else
					pReceiver->UngetRcvBuff(pbBuff);
				pReceiver->m_fRestart = FALSE;
				return 0;
			case (WSA_WAIT_EVENT_0 + 1): // receive operation is complete
				if (!WSAGetOverlappedResult(pReceiver->m_hSocket, &Overlapped, &cbReceived, FALSE, &dwFlags))
					if (pReceiver->m_fRestart)
						goto error1;
					else
						goto error;
				break;
			}
		}
		else
		{
			if (pReceiver->m_fRestart)
				goto error1;
			else
				goto error;
		}
		*(LPDWORD)pbBuff = Buff.len;

		if (pReceiver->m_fRestart)
		{
			if (*(LPDWORD)pbBuff == MPEG_PROGRAM_END_CODE)
			{
				pReceiver->m_fRestart = FALSE;
				delete[] pbBuff;
				PostMessage(pReceiver->m_pReceiverWnd->m_hWnd, MSG_DEC_STOP, pReceiver->m_nId, 0);
			}

			//MpPbg͂VXewb_ĐĊJ
			//check MPEG2 program stream pack and system header
			if (pReceiver->CheckPacket(pbBuff, *(LPDWORD)pbBuff))
			{
				DEC_STATUS dec_status;
				DEC_RETURN dec_return;
				dec_return = DEC_Get_Status(pReceiver->m_nId, &dec_status);
				if (dec_return == DEC_SUCCESS && dec_status == DEC_STATUS_STOP)
				{
					pReceiver->m_eventStop.ResetEvent();
					//ĐĊJ
					//starting to play
					if (DEC_Play(pReceiver->m_nId)==DEC_SUCCESS)
					{
#ifdef __TRACEOUT
						TraceOut("DEC_Play Successful");
#endif
						LPBYTE lpBuff = pReceiver->GetRcvBuff();
						if (lpBuff)
						{
							memcpy(lpBuff, pbBuff, pReceiver->m_cbBuff + sizeof(DWORD));
							pReceiver->PutBsrBuff(lpBuff);
						}
						pReceiver->m_fRestart = FALSE;
						pReceiver->m_fStopReq = FALSE;
						fBuffEmpty = FALSE;
					}
					else
					{
						uError = DEC_Get_Last_Error(pReceiver->m_nId);
#ifdef __TRACEOUT
						char szTemp[32];
						wsprintf(szTemp, "DEC_Play Error Code: %d", uError);
						TraceOut(szTemp);
#endif
						goto error1;
					}
				}
			}
			delete[] pbBuff;
		}
		else
		{
			if (*(LPDWORD)pbBuff != MPEG_PROGRAM_END_CODE)
			{
				pReceiver->CheckStcValue(pbBuff, *(LPDWORD)pbBuff);
			}
			pReceiver->PutBsrBuff(pbBuff);
		}
	}
error: // error occurred
#ifdef __TRACEOUT
	TraceOut("Thread Error (0)");
#endif
	pReceiver->m_fRestart = FALSE;
	pReceiver->UngetRcvBuff(pbBuff);
	pReceiver->m_pReceiverWnd->PostMessage(MSG_DEC_ERROR, pReceiver->m_nId, ERROR_THREAD);
	return (UINT)-1;
error1:
#ifdef __TRACEOUT
	TraceOut("Thread Error (1)");
#endif
	pReceiver->m_fRestart = FALSE;
	delete[] pbBuff;
	pReceiver->m_pReceiverWnd->PostMessage(MSG_DEC_ERROR, pReceiver->m_nId, ERROR_THREAD);
	return (UINT)-1;
}

BOOL CMpegReceiver::BsrCallback(LPBYTE pbBuff, DWORD cbWrite, LPDWORD pcbWritten)
{
	LPBYTE pbSrc = GetBsrBuff();

	if (pbSrc == NULL)
	{
		*pcbWritten = 0;
		return FALSE;
	}

	if (m_fState == trsStarting)
		m_fState = trsStarted;
	*pcbWritten = *(LPDWORD)pbSrc;
	memcpy(pbBuff, pbSrc + sizeof(DWORD), *pcbWritten);
	if (*(LPDWORD)pbBuff == MPEG_PROGRAM_END_CODE)
	{ // last data
		*(LPDWORD)pbBuff = 0;
		*pcbWritten = 0;
		UngetBsrBuff(pbSrc);
		if (m_fState == trsStarted)
		{
			m_fState = trsStopping;
			return TRUE;
		}
		else
			return FALSE;
	}
	else
	{
		PutRcvBuff(pbSrc);
		return FALSE;
	}
}

LPBYTE CMpegReceiver::GetRcvBuff()
{
	LPBYTE pbBuff = NULL;
	m_csec.Lock();
	if (!m_listRcvBuffs.IsEmpty())
		pbBuff = m_listRcvBuffs.RemoveHead();
	m_csec.Unlock();
	return pbBuff;
}

void CMpegReceiver::UngetRcvBuff(LPBYTE pbBuff)
{
	m_csec.Lock();
	m_listRcvBuffs.AddHead(pbBuff);
	m_csec.Unlock();
}

void CMpegReceiver::PutRcvBuff(LPBYTE pbBuff)
{
	m_csec.Lock();
	m_listRcvBuffs.AddTail(pbBuff);
	m_csec.Unlock();
	m_eventRcv.SetEvent();
}

// We assume that CMpegReceiver::GetBsrBuff and CMpegReceiver::PutBsrBuff will be called in the context of 2 different threads.

LPBYTE CMpegReceiver::GetBsrBuff()
{
	LPBYTE pbBuff = NULL;
	HANDLE hEvents[2];
	hEvents[0] = m_eventStop.m_hObject;
	hEvents[1] = m_eventBsr.m_hObject;
	while (TRUE) {
		m_csec.Lock();
		if (!m_listBsrBuffs.IsEmpty())
			pbBuff = m_listBsrBuffs.RemoveHead();
		m_csec.Unlock();
		if (pbBuff)
			break;

		if (WaitForMultipleObjects(sizeof(hEvents)/sizeof(HANDLE), hEvents, FALSE, INFINITE) - WAIT_OBJECT_0 == 0)
		{
			pbBuff = NULL;
			break;
		}
	}
	return pbBuff;
}

void CMpegReceiver::UngetBsrBuff(LPBYTE pbBuff)
{
	m_csec.Lock();
	m_listBsrBuffs.AddHead(pbBuff);
	m_csec.Unlock();
}

void CMpegReceiver::PutBsrBuff(LPBYTE pbBuff)
{
	m_csec.Lock();
	m_listBsrBuffs.AddTail(pbBuff);
	m_csec.Unlock();
	m_eventBsr.SetEvent();
}

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverThread

IMPLEMENT_DYNCREATE(CMpegReceiverThread, CWinThread)

CMpegReceiverThread::CMpegReceiverThread()
{
}

CMpegReceiverThread::~CMpegReceiverThread()
{
}

BOOL CMpegReceiverThread::InitInstance()
{
	// create an invisible window (just a message router)
	m_pMainWnd = new CMpegReceiverWnd;
	if (!m_pMainWnd->CreateEx(0, AfxRegisterWndClass(0), _T(""), WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL)) {
		delete m_pMainWnd; m_pMainWnd = NULL;
		return FALSE;
	}
	*(CMpegReceiverWnd**)m_pThreadParams = (CMpegReceiverWnd*)m_pMainWnd;
	return TRUE;
}

int CMpegReceiverThread::ExitInstance()
{
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(CMpegReceiverThread, CWinThread)
	//{{AFX_MSG_MAP(CMpegReceiverThread)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂B
		// NOTE - the ClassWizard will add mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverThread bZ[W nh
// CMpegReceiverThread message handlers

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverWnd

CMpegReceiverWnd::CMpegReceiverWnd()
{
}

CMpegReceiverWnd::~CMpegReceiverWnd()
{
}

BEGIN_MESSAGE_MAP(CMpegReceiverWnd, CWnd)
	//{{AFX_MSG_MAP(CMpegReceiverWnd)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂͍폜܂B
		// NOTE - the ClassWizard will add and remove mapping macros here.
	ON_WM_CLOSE()
	ON_MESSAGE(MSG_DEC_STATUS, OnDecStatus)
	ON_MESSAGE(MSG_DEC_ERROR, OnDecError)
	ON_MESSAGE(MSG_DEC_STOP, OnDecStop)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverWnd bZ[W nh
// CMpegReceiverWnd message handlers


void CMpegReceiverWnd::OnClose()
{
	CWnd::OnClose();
	PostQuitMessage(0);
}

void CMpegReceiverWnd::PostNcDestroy()
{
	CWnd::PostNcDestroy();
	delete this;
}

LRESULT CMpegReceiverWnd::OnDecStatus(WPARAM wParam, LPARAM lParam)
{
	CDecCallback *pCallback = CMpegReceiver::FindDecCallback((UINT)wParam);
	if (!pCallback)
		return 0;

#ifdef __TRACEOUT
	char szTemp[32];
	wsprintf(szTemp, "DecStatus Callback - Status: %d", lParam);
	TraceOut(szTemp);
#endif

	switch ((DEC_STATUS_NOTIFY)lParam) {
	case DEC_NOTIFY_PLAY:
#ifdef __TRACEOUT
		g_nDecID = pCallback->m_nDecId;
		g_nTimerID = ::SetTimer(NULL, NULL, 60 * 1000, LogOutTimerProc);
#endif
		pCallback->m_pReceiver->m_nDiff = 0;
		pCallback->m_pReceiver->m_fCheckStc = FALSE;
		pCallback->m_pReceiver->m_eventStopped.ResetEvent();
		break;
	case DEC_NOTIFY_STOP:
		pCallback->m_pReceiver->m_eventStopped.SetEvent();
#ifdef __TRACEOUT
		::KillTimer(NULL, g_nTimerID);
#endif
		pCallback->m_pReceiver->m_nDiff = 0;
		pCallback->m_pReceiver->m_fCheckStc = FALSE;
#ifdef __TRACEOUT
		g_bLogOut = FALSE;
#endif
		ASSERT(pCallback->m_pReceiver);
		if (pCallback->m_pReceiver->m_fRestart)
			return 0;
		pCallback->m_pReceiver->OnDecStopped();
		break;
	default:
		break;
	}
	if (pCallback->m_pfnStatus)
		(*(pCallback->m_pfnStatus))((UINT)wParam, (DEC_STATUS_NOTIFY)lParam);
	return 0;
}

LRESULT CMpegReceiverWnd::OnDecError(WPARAM wParam, LPARAM lParam)
{
	CDecCallback *pCallback = CMpegReceiver::FindDecCallback((UINT)wParam);
	if (!pCallback)
		return 0;
	if (pCallback->m_pReceiver)
		pCallback->m_pReceiver->ReceiveStop();
	if (pCallback->m_pfnError)
		(*(pCallback->m_pfnError))((UINT)wParam, (ULONG)lParam);
	return 0;
}

LRESULT CMpegReceiverWnd::OnDecStop(WPARAM wParam, LPARAM lParam)
{
	CDecCallback *pCallback = CMpegReceiver::FindDecCallback((UINT)wParam);
	if (!pCallback)
		return 0;
	if (pCallback->m_pReceiver)
	{
		pCallback->m_pReceiver->TerminateReceiveThread();
		pCallback->m_pReceiver->m_fRestart = FALSE;
	}
	return 0;
}

BOOL CMpegReceiver::CheckPacket(LPBYTE lpBuff, DWORD dwSize)
{
	if (dwSize < 2048) return FALSE;

	BOOL bFind = FALSE;
	LPBYTE lpPointer = lpBuff + 4; 
	UINT nSize = 0;
	while (nSize < dwSize)
	{
		//pack start code
		LPDWORD lpPSC = (LPDWORD)lpPointer;
		if (*lpPSC == 0xBA010000)
		{
			BYTE bSSize = (BYTE)*(lpPointer + 13);
			bSSize &= 0x7;

			//confirm system header start code
			LPDWORD lpSHSC = (LPDWORD)(lpPointer + 14 + bSSize);
			if (*lpSHSC == 0xBB010000)
			{
				bFind = TRUE;
				break;
			}
		}
		lpPointer += 2048;
		nSize += 2048;
	}
	return bFind;
}

#ifdef __TRACEOUT
VOID CALLBACK LogOutTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	DEC_STATUS dec_status;
	if (DEC_Get_Status(g_nDecID, &dec_status)==DEC_SUCCESS)
	{
		g_bLogOut = TRUE;
	}
}
#endif

#define DIFF_VALUE_REVISE 0.8 // x < 1
void CMpegReceiver::CheckStcValue(LPBYTE lpBuff, DWORD dwSize)
{
	if (dwSize < 2048) return;

	BOOL bFind = FALSE;
	LPBYTE lpPointer = lpBuff + 4; 
	UINT nSize = 0;
	while (nSize < dwSize)
	{
		//pack start code 
		LPDWORD lpPSC = (LPDWORD)lpPointer;
		if (*lpPSC == 0xBA010000)
		{
			BYTE bSSize = (BYTE)*(lpPointer + 13);
			bSSize &= 0x7;

			//confirm system header start code
			LPDWORD lpSHSC = (LPDWORD)(lpPointer + 14 + bSSize);
			if (*lpSHSC == 0xBB010000)
			{
				bFind = TRUE;
				break;
			}
		}
		lpPointer += 2048;
		nSize += 2048;
	}
	if (!bFind)	return;

	//not check first system header
	if (!m_fCheckStc)
	{
		m_fCheckStc = TRUE;
		return;
	}

	//calclate average 
	LPBYTE lpSCR = lpPointer + 4;
	DWORD dwSTCValue = 0;
	DWORD dwSCRValue = 0;
	if (DEC_Get_Sync_Stc_Value(m_nId, &dwSTCValue)==DEC_SUCCESS)
	{
		//MPEG2 SCR value
		dwSCRValue = ((*(lpSCR+1)&0xF)<<12) + ((*(lpSCR+2)&0xF8)<<4) + ((*(lpSCR+2)&0x3)<<5) + ((*(lpSCR+3)&0xF8)>>3);
		int	 nDiff = dwSCRValue - dwSTCValue; //difference STC from SCR 

		if (nDiff < -32768)
			nDiff += 65536;
		if (nDiff > 32767)
			nDiff -= 65536;

		if (m_nDiff == 0)
			m_nDiff = int(nDiff/(1 - DIFF_VALUE_REVISE));
		else
			m_nDiff = int(nDiff + (m_nDiff * DIFF_VALUE_REVISE));
		
		if (int((1 - DIFF_VALUE_REVISE) * m_nDiff) < 32)
		{
#ifdef __TRACEOUT
			char szTemp[64];
			wsprintf(szTemp, "STC: 0x%X, SCR: 0x%X, Diff: 0x%X, Average: 0x%X", dwSTCValue, dwSCRValue, nDiff, int((1-DIFF_VALUE_REVISE) * m_nDiff));
			TraceOut(szTemp);
#endif
			m_fStopReq = TRUE;
		}

#ifdef __TRACEOUT
		if (!g_bLogOut) return;

		char szTemp[64];
		wsprintf(szTemp, "STC: 0x%X, SCR: 0x%X, Diff: 0x%X, Average: 0x%X", dwSTCValue, dwSCRValue, nDiff, int((1-DIFF_VALUE_REVISE) * m_nDiff));
		TraceOut(szTemp);

		g_bLogOut = FALSE;
#endif
	}
}
