
// MpegReceiver.h : MPEG program stream receiver classes definitions
// Copyright (C) 2005 Canopus Co., Ltd. All rights reserved.

#include <afxmt.h>
#include <afxtempl.h>
#include "MpegTransfer.h"

// for forward references
class CMpegReceiver;
class CMpegReceiverThread;
class CMpegReceiverWnd;

// private messages
#define MSG_DEC_STATUS		WM_APP
#define MSG_DEC_ERROR		(WM_APP + 1)
#define MSG_DEC_STOP		(WM_APP + 2)

#define ERROR_THREAD		1024

#define MPEG_PROGRAM_END_CODE	0xb9010000 // MPEG_program_end_code in reverse order

/////////////////////////////////////////////////////////////////////////////
// helpers

struct CDecCallback {
	UINT m_nDecId;
	CMpegReceiver* m_pReceiver;
	DEC_CB_STATUS m_pfnStatus;
	DEC_CB_ERROR m_pfnError;

	CDecCallback(UINT nDecId, CMpegReceiver* pReceiver, DEC_CB_STATUS pfnStatus, DEC_CB_ERROR pfnError);
	~CDecCallback();
};

inline CDecCallback::CDecCallback(UINT nDecId, CMpegReceiver* pReceiver, DEC_CB_STATUS pfnStatus, DEC_CB_ERROR pfnError)
	: m_nDecId(nDecId), m_pReceiver(pReceiver), m_pfnStatus(pfnStatus), m_pfnError(pfnError)
	{}
inline CDecCallback::~CDecCallback()
	{}

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiver

class CMpegReceiver : public CMpegTransfer
{
	DECLARE_DYNAMIC(CMpegReceiver)

// attributes
protected:
	static CCriticalSection m_csec; // for exclusive access to m_listDecCallbacks
	static CTypedPtrList<CPtrList, CDecCallback*> m_listDecCallbacks;
	static CCriticalSection m_csecThread; // for exclusive access to m_pReceiverThread
	static CMpegReceiverThread* m_pReceiverThread;
	static CMpegReceiverWnd* m_pReceiverWnd;

	WSAEVENT m_hEventStop;
	CEvent m_eventRcv;
	CEvent m_eventBsr;
	CTypedPtrList<CPtrList, LPBYTE> m_listRcvBuffs;
	CTypedPtrList<CPtrList, LPBYTE> m_listBsrBuffs;
	CWinThread* m_pReceiveThread;
#ifdef _DEBUG
	DWORD m_cRcvBuffMinimum; // min. receive buffer count
#endif

public:
	BOOL	m_fRestart;
	BOOL	m_fStopReq;
	CEvent	m_eventStop;
	CEvent	m_eventStopped;
	int		m_nDiff;
	BOOL	m_fCheckStc;

// operations
protected:
	static BOOL RegisterReceiver(UINT nDecId, CMpegReceiver* pReceiver);
	static void CALLBACK DecStatusCallback(UINT nDecId, DEC_STATUS_NOTIFY fStatus);
	static void CALLBACK DecErrorCallback(UINT nDecId, ULONG ulError);
	static void CALLBACK BsrCallback(LPBYTE pbBuff, DWORD cbWrite, LPDWORD pcbWritten, DWORD dwData);

	void DiscardBuffs();
	void TerminateReceiveThread();
	BOOL DecodeStop();
	LPBYTE GetRcvBuff();
	void UngetRcvBuff(LPBYTE pbBuff);
	void PutRcvBuff(LPBYTE pbBuff);
	LPBYTE GetBsrBuff();
	void UngetBsrBuff(LPBYTE pbBuff);
	void PutBsrBuff(LPBYTE pbBuff);

public:
	CMpegReceiver(UINT nDecId);
	virtual ~CMpegReceiver();
	static BOOL Init(UINT nDecId, CMpegReceiver* pReceiver = NULL, DEC_CB_STATUS pfnStatus = NULL, DEC_CB_ERROR pfnError = NULL);
	static BOOL Exit(UINT nDecId);
	static CDecCallback* FindDecCallback(UINT nDecId);

	BOOL Init(DEC_CB_STATUS pfnStatus = NULL, DEC_CB_ERROR pfnError = NULL);
	BOOL Exit();
	virtual BOOL Connect(LPCTSTR pszAddress, WORD nPort, LPCTSTR pszAddressOpposite);
	virtual BOOL Disconnect();
	BOOL ReceiveStart(DWORD cbBssBuff = m_cbBuffDefault, DWORD cBssBuff = m_cBuffDefault);
	BOOL ReceiveStop();
	void OnDecStopped();
	BOOL BsrCallback(LPBYTE pbBuff, DWORD cbWrite, LPDWORD pcbWritten);

	//STC-REG check functions
	BOOL CheckPacket(LPBYTE lpBuff, DWORD dwSize);
	void CheckStcValue(LPBYTE lpBuff, DWORD dwSize);

	friend UINT ReceiveThread(LPVOID pParam);
	friend CMpegReceiverWnd;
};

inline BOOL CMpegReceiver::Init(DEC_CB_STATUS pfnStatus /* = NULL */, DEC_CB_ERROR pfnError /* = NULL */)
	{return Init(m_nId, this, pfnStatus, pfnError);}

inline BOOL CMpegReceiver::Exit()
	{return Exit(m_nId);}

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverThread

class CMpegReceiverThread : public CWinThread
{
	DECLARE_DYNCREATE(CMpegReceiverThread)
protected:
	CMpegReceiverThread();          // IɎgpveNg RXgN^
									// protected constructor used by dynamic creation 

// Agr[g
// Attributes
public:

// Iy[V
// Operations
public:
	virtual ~CMpegReceiverThread();

// I[o[Ch
// Overrides
	// ClassWizard ͉z֐̃I[o[Ch𐶐܂B
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMpegReceiverThread)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Cve[V
// Implementation
protected:

	// ꂽbZ[W }bv֐
	// Generated message map functions
	//{{AFX_MSG(CMpegReceiverThread)
		//  - ClassWizard ͂̈ʒuɃo֐ǉ܂͍폜܂B
		// NOTE: the ClassWizard will add member functions here
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CMpegReceiverWnd

class CMpegReceiverWnd : public CWnd
{
// RXgNV
// Construction
public:
	CMpegReceiverWnd();

// Agr[g
// Attributes
public:

// Iy[V
// Operations
public:

// I[o[Ch
// Overrides
	// ClassWizard ͉z֐̃I[o[Ch𐶐܂B
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMpegReceiverWnd)
	protected:
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

// Cve[V
// Implementation
public:
	virtual ~CMpegReceiverWnd();

protected:
	UINT m_nTimer; // timer identifier

	// ꂽbZ[W }bv֐
	// Generated message map functions
protected:
	//{{AFX_MSG(CMpegReceiverWnd)
		//  - ClassWizard ͂̈ʒuɃo֐ǉ܂͍폜܂B
		// NOTE: the ClassWizard will add member functions here
	afx_msg void OnClose();
	afx_msg LRESULT OnDecStatus(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnDecError(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnDecStop(WPARAM wParam, LPARAM lParam);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

