
// MpegSender.h : MPEG program stream sender classes definitions
// Copyright (C) 2005 Canopus Co., Ltd. All rights reserved.

#include "stdafx.h"
#include "MpegSender.h"

/////////////////////////////////////////////////////////////////////////////
// CMpegSender

IMPLEMENT_DYNAMIC(CMpegSender, CMpegTransfer)

/////////////////////////////////////////////////////////////////////////////
// CMpegSender constructor / destructor

CMpegSender::CMpegSender(UINT nEncId)
	: CMpegTransfer(nEncId), m_pbBssBuffAux(NULL)
{
	RegisterSender(m_nId, this);
	m_pbBssBuffAux = new BYTE[m_cbBuffDefault];

	m_fRestarting = FALSE;
}

CMpegSender::~CMpegSender()
{
	SendStop();
	Disconnect();
	RegisterSender(m_nId, NULL);
	delete[] m_pbBssBuffAux; m_pbBssBuffAux = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// CMpegSender static members

CCriticalSection CMpegSender::m_csec; // for exclusive access to m_listEncCallbacks
CTypedPtrList<CPtrList, CEncCallback*> CMpegSender::m_listEncCallbacks;
CCriticalSection CMpegSender::m_csecThread; // for exclusive access to m_pSenderThread
CMpegSenderThread* CMpegSender::m_pSenderThread = NULL;
CMpegSenderWnd* CMpegSender::m_pSenderWnd = NULL;

BOOL CMpegSender::Init(UINT nEncId, CMpegSender* pSender /* = NULL */,
	ENC_CB_STATUS pfnStatus /* = NULL */, ENC_CB_ERROR pfnError /* = NULL */, ENC_CB_VOBU_ENT pfnVobuEnt /* = NULL */)
{
	BOOL fResult = FALSE;
	m_csecThread.Lock();
	if (!m_pSenderThread && (m_pSenderThread = (CMpegSenderThread*)AfxBeginThread(RUNTIME_CLASS(CMpegSenderThread),
		THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED))) { // one time initialization
		m_pSenderThread->m_bAutoDelete = FALSE; m_pSenderThread->m_pThreadParams = &m_pSenderWnd;
		VERIFY(m_pSenderThread->ResumeThread() == 1);
	}
	m_csecThread.Unlock();
	if (!m_pSenderThread)
		goto done;
	m_csec.Lock();
	m_listEncCallbacks.AddTail(new CEncCallback(nEncId, NULL, pfnStatus, pfnError, pfnVobuEnt));
	m_csec.Unlock();
	if (ENC_Set_Callback(nEncId, EncStatusCallback, EncErrorCallback, EncVobuCallback, NULL, NULL) != ENC_SUCCESS)
		goto done;
	ENC_BSS_PARAMETER Param;
	Param.fType = ENC_BSS_TYPE_PS;
	Param.pfnCallback = BssCallback;
	Param.cbBuff = m_cbBuffDefault; Param.cBuff = m_cBuffDefault; Param.dwData = (DWORD)pSender;
	if (ENC_Initialize_Ex(nEncId, ENC_MEDIA_MEMORY, &Param) != ENC_SUCCESS)
		goto done;

	//GOP Complete
	ENC_VIDEO_ENCODE_PARAMETER_EX enc_param_ex;
	if (ENC_Get_Video_Encode_Parameter_Ex(nEncId, &enc_param_ex) != ENC_SUCCESS)
		goto done;

	enc_param_ex.fGopComplete = TRUE;
	
	if (ENC_Set_Video_Encode_Parameter_Ex(nEncId, &enc_param_ex) != ENC_SUCCESS)
		goto done;
	fResult = TRUE;
done:
	if (!fResult)
		Exit(nEncId);
	return fResult;
}

BOOL CMpegSender::Exit(UINT nEncId)
{
	VERIFY(ENC_Terminate(nEncId) == ENC_SUCCESS);
	m_csec.Lock();
	POSITION pos, posPrev;
	for (pos = m_listEncCallbacks.GetHeadPosition(); posPrev = pos; ) {
		CEncCallback* pCallback = m_listEncCallbacks.GetNext(pos);
		if (pCallback->m_nEncId == nEncId) {
			m_listEncCallbacks.RemoveAt(posPrev); delete pCallback;
			break;
		}
	}
	m_csec.Unlock();
 	if (!posPrev)
		goto done;
	m_csecThread.Lock();
	if (m_listEncCallbacks.IsEmpty() && m_pSenderThread) { // one time termination
		DWORD dwExitCode;
		VERIFY(GetExitCodeThread(m_pSenderThread->m_hThread, &dwExitCode));
		if (dwExitCode == STILL_ACTIVE) {
			ASSERT(IsWindow(m_pSenderWnd->GetSafeHwnd()));
			 // request sender thread to terminate itself, and wait for it
			VERIFY(m_pSenderWnd->PostMessage(WM_CLOSE, 0, 0));
			WaitForSingleObject(m_pSenderThread->m_hThread, INFINITE);
			delete m_pSenderThread; m_pSenderThread = NULL;
			m_pSenderWnd = NULL;
		}
	}
	m_csecThread.Unlock();
done:
	return TRUE;
}

BOOL CMpegSender::RegisterSender(UINT nEncId, CMpegSender* pSender) // register / unregister CMpegSender instance
{
	m_csec.Lock();
	for (POSITION pos = m_listEncCallbacks.GetHeadPosition(); pos; ) {
		CEncCallback* pCallback = m_listEncCallbacks.GetNext(pos);
		if (pCallback->m_nEncId == nEncId) {
			pCallback->m_pSender = pSender;
			m_csec.Unlock();
			return TRUE;
		}
	}
	m_csec.Unlock();
	return FALSE;
}

CEncCallback* CMpegSender::FindEncCallback(UINT nEncId)
{
	m_csec.Lock();
	for (POSITION pos = m_listEncCallbacks.GetHeadPosition(); pos; ) {
		CEncCallback* pCallback = m_listEncCallbacks.GetNext(pos);
		if (pCallback->m_nEncId == nEncId) {
			m_csec.Unlock();
			return pCallback;
		}
	}
	m_csec.Unlock();
	return NULL;
}

// the following callbacks may be called in the context of the thread other than the one for theApp

void CALLBACK CMpegSender::EncStatusCallback(UINT nEncId, ENC_STATUS_NOTIFY fStatus)
{
	if (m_pSenderWnd)
		m_pSenderWnd->PostMessage(MSG_ENC_STATUS, (WPARAM)nEncId, (LPARAM)fStatus);
}

void CALLBACK CMpegSender::EncErrorCallback(UINT nEncId, ULONG ulError)
{
	if (m_pSenderWnd)
		m_pSenderWnd->PostMessage(MSG_ENC_ERROR, (WPARAM)nEncId, (LPARAM)ulError);
}

void CALLBACK CMpegSender::EncVobuCallback(UINT nEncId, ENC_VOBU_ENT_INFO* pInfo, UINT fBoundaryFlag)
{
	CEncCallback* pCallback = FindEncCallback(nEncId);
	if (pCallback && pCallback->m_pfnVobuEnt)
		(*(pCallback->m_pfnVobuEnt))(nEncId, pInfo, fBoundaryFlag);
}

void CALLBACK CMpegSender::BssCallback(LPBYTE pbBuff, DWORD cbBuff, DWORD dwData)
{
	if (dwData) {
		CMpegSender* pSender = (CMpegSender*)dwData;
		pSender->BssCallback(pbBuff, cbBuff);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMpegSender members

BOOL CMpegSender::SendStart(DWORD cbBssBuff /* = m_cbBuffDefault */, DWORD cBssBuff /* = m_cBuffDefault */)
{
	m_fStarting = TRUE;
	m_fRestarting = FALSE;

	ASSERT(m_fState == trsConnected || m_fState == trsStopped);
	m_cbBuff = cbBssBuff; m_cBuff = cBssBuff;
// We should flush the current contents of the sending queue,
// but unfortunately, WSAIoctl with control code SIO_FLUSH does not seem to work.
// The receiver often failes to receive the head portion of sent data. To workaround this problem, send some dummy data here.
	DWORD dw = 0;
	for (int i = 0; i < 3; i++)
		if (BssCallback((LPBYTE)&dw, sizeof(DWORD)))
			goto done;
	ENC_BSS_PARAMETER BssParam;
	if (ENC_Can_Record(m_nId) != ENC_SUCCESS)
		goto done;
	if (ENC_Get_BSS_Parameter(m_nId, &BssParam) != ENC_SUCCESS)
		goto done;
	BssParam.cbBuff = m_cbBuff; BssParam.cBuff = m_cBuff; BssParam.dwData = (DWORD)this;
	if (ENC_Set_BSS_Parameter(m_nId, &BssParam) != ENC_SUCCESS)
		goto done;
	if (ENC_Set_Record_Time(m_nId, 0, FALSE) != ENC_SUCCESS)
		goto done;
	if (ENC_Init_Movie(m_nId) != ENC_SUCCESS || ENC_Record_Movie(m_nId, ENC_RECORD_PS) != ENC_SUCCESS)
		goto done;
	m_fState = trsStarted;
done:
	m_fStarting = FALSE;
	if (m_fState == trsStarted)
	{
		m_pSenderWnd->m_bRestartReq = TRUE;
		return TRUE;
	}
	else {
		SendStop();
		return FALSE;
	}
}

BOOL CMpegSender::SendStop()
{
	m_pSenderWnd->m_bRestartReq = FALSE;

#ifdef __TRACEOUT
	TraceOut("Send Stop");
#endif

	if (m_fState == trsStarted) {
		if (ENC_Stop(m_nId) != ENC_SUCCESS)
			return FALSE;
		m_fState = trsStopped;
	}
	return TRUE;
}

int CMpegSender::BssCallback(LPBYTE pbBuff, DWORD cbBuff)
{
	int nError;
	ASSERT(cbBuff <= m_cbBuff);
	if (cbBuff < m_cbBuff) { // if data length is less than m_cbBuff (i.e, last data), use auxiliary buffer
		memcpy(m_pbBssBuffAux, pbBuff, cbBuff);
		pbBuff = m_pbBssBuffAux;
		memset(pbBuff + cbBuff, 0, m_cbBuff - cbBuff);
	}

	VERIFY(WSAResetEvent(m_hEvent));
	WSABUF Buff;
	Buff.len = m_cbBuff; Buff.buf = (LPSTR)pbBuff;
	WSAOVERLAPPED Overlapped;
	Overlapped.hEvent = m_hEvent;
	DWORD cbSent = 0, dwFlags = 0;
	int nResult = WSASendTo(m_hSocket, &Buff, 1, &cbSent, dwFlags,
		(LPSOCKADDR)&m_AddrOpposite, sizeof(SOCKADDR_IN), &Overlapped, NULL);
	nError = WSAGetLastError();
	if (!nResult) // operation is completed
	{
		ASSERT(cbSent == m_cbBuff && !dwFlags);
	}
	else if (nError == WSA_IO_PENDING && WSAWaitForMultipleEvents(1, &m_hEvent, FALSE, INFINITE, FALSE) == WAIT_OBJECT_0 &&
			WSAGetOverlappedResult(m_hSocket, &Overlapped, &cbSent, FALSE, &dwFlags))
	{
		nResult = 0;
		ASSERT(cbSent == m_cbBuff && !dwFlags);
	}
	if (nResult)
	{
#ifdef __TRACEOUT
		char szTemp[64];
		wsprintf(szTemp, "BssCallback Error - WSAGetLastError:%d", nError);
		TraceOut(szTemp);
#endif
		CEncCallback *pCallback = CMpegSender::FindEncCallback(m_nId);
		if (pCallback->m_pfnError)
		{
			m_fRestarting = TRUE;
			(*(pCallback->m_pfnError))(m_nId, SOCKET_ERROR);
		}
	}
	return nResult;
}

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderThread

IMPLEMENT_DYNCREATE(CMpegSenderThread, CWinThread)

CMpegSenderThread::CMpegSenderThread()
{
}

CMpegSenderThread::~CMpegSenderThread()
{
}

BOOL CMpegSenderThread::InitInstance()
{
	// create an invisible window (just a message router)
	m_pMainWnd = new CMpegSenderWnd;
	if (!m_pMainWnd->CreateEx(0, AfxRegisterWndClass(0), _T(""), WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL)) {
		delete m_pMainWnd; m_pMainWnd = NULL;
		return FALSE;
	}
	*(CMpegSenderWnd**)m_pThreadParams = (CMpegSenderWnd*)m_pMainWnd;
	return TRUE;
}

int CMpegSenderThread::ExitInstance()
{
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(CMpegSenderThread, CWinThread)
	//{{AFX_MSG_MAP(CMpegSenderThread)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂B
		// NOTE - the ClassWizard will add mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderThread bZ[W nh
// CMpegSenderThread message handlers

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderWnd

CMpegSenderWnd::CMpegSenderWnd()
{
	m_bRestartReq = FALSE;
}

CMpegSenderWnd::~CMpegSenderWnd()
{
}

BEGIN_MESSAGE_MAP(CMpegSenderWnd, CWnd)
	//{{AFX_MSG_MAP(CMpegSenderWnd)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂͍폜܂B
		// NOTE - the ClassWizard will add and remove mapping macros here.
	ON_WM_CLOSE()
	ON_MESSAGE(MSG_ENC_STATUS, OnEncStatus)
	ON_MESSAGE(MSG_ENC_ERROR, OnEncError)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderWnd bZ[W nh
// CMpegSenderWnd message handlers

void CMpegSenderWnd::OnClose()
{
	CWnd::OnClose();
	PostQuitMessage(0);
}

void CMpegSenderWnd::PostNcDestroy()
{
	CWnd::PostNcDestroy();
	delete this;
}

LRESULT CMpegSenderWnd::OnEncStatus(WPARAM wParam, LPARAM lParam)
{
#ifdef __TRACEOUT
	char szTemp[32];
	wsprintf(szTemp, "Status CallBack\tCode: %d", lParam);
	TraceOut(szTemp);
#endif
	CEncCallback *pCallback = CMpegSender::FindEncCallback((UINT)wParam);
	if (!pCallback)
		return 0;
	switch ((ENC_STATUS_NOTIFY)lParam) {
	case ENC_NOTIFY_STOP:
	{
		//ċNԂł͍ŏIR[h𑗂Ȃ
		//if restarting, not send end code
		if (!m_bRestartReq || pCallback->m_pSender->m_fStarting)
		{
			DWORD dw = MPEG_PROGRAM_END_CODE;
			ASSERT(pCallback->m_pSender);
			pCallback->m_pSender->BssCallback((LPBYTE)&dw, sizeof(DWORD)); // send last data which only includes MPEG_program_end_code
		}
		else if (!pCallback->m_pSender->m_fRestarting)
		{
#ifdef __TRACEOUT
			TraceOut("CMpegSenderWnd - Restarting encoder...");
#endif
			ENC_BSS_PARAMETER BssParam; 
			if (ENC_Can_Record(pCallback->m_pSender->m_nId) != ENC_SUCCESS)
				break;
			if (ENC_Get_BSS_Parameter(pCallback->m_pSender->m_nId, &BssParam) != ENC_SUCCESS)
				break;
			BssParam.cbBuff = pCallback->m_pSender->m_cbBuff; BssParam.cBuff = pCallback->m_pSender->m_cBuff; BssParam.dwData = (DWORD)pCallback->m_pSender;
			if (ENC_Set_BSS_Parameter(pCallback->m_pSender->m_nId, &BssParam) != ENC_SUCCESS)
				break;
			if (ENC_Set_Record_Time(pCallback->m_pSender->m_nId, 0, FALSE) != ENC_SUCCESS)
				break;
			if (ENC_Init_Movie(pCallback->m_pSender->m_nId) != ENC_SUCCESS || ENC_Record_Movie(pCallback->m_pSender->m_nId, ENC_RECORD_PS) != ENC_SUCCESS)
				break;
#ifdef __TRACEOUT
			TraceOut("CMpegSenderWnd - Restarting encoder done.");
#endif
		}
		break;
	}
	default:
		break;
	}
	if (!m_bRestartReq)
	{
		if (pCallback->m_pfnStatus)
			(*(pCallback->m_pfnStatus))((UINT)wParam, (ENC_STATUS_NOTIFY)lParam);
	}
	return 0;
}

LRESULT CMpegSenderWnd::OnEncError(WPARAM wParam, LPARAM lParam)
{
	CEncCallback *pCallback = CMpegSender::FindEncCallback((UINT)wParam);
	if (!pCallback)
		return 0;

	if (pCallback->m_pSender) return 0;
#ifdef __TRACEOUT
	char szTemp[32];
	wsprintf(szTemp, "Error Callback\tCode: %d", lParam);
	TraceOut(szTemp);
#endif

	ULONG ulError = lParam;
	if (ulError == MVR_ERROR_EVENT_DATA_OVERRUN ||
		ulError == MVR_ERROR_EVENT_LOSS_OF_VIDEO)
	{
		if (!pCallback->m_pSender->m_fStarting)
		{
#ifdef __TRACEOUT
			TraceOut("Stopping encoder...");
#endif
			m_bRestartReq = TRUE;
			if (ENC_Stop(pCallback->m_pSender->m_nId) != ENC_SUCCESS)
				goto error;
#ifdef __TRACEOUT
			TraceOut("Stopping encoder done.");
#endif
		}
		return 0;
	}
	
error:
	pCallback->m_pSender->SendStop();
	if (pCallback->m_pfnError)
		(*(pCallback->m_pfnError))((UINT)wParam, (ULONG)lParam);
	return 0;
}