
// MpegSender.h : MPEG program stream sender classes definitions
// Copyright (C) 2005 Canopus Co., Ltd. All rights reserved.

#include <afxmt.h>
#include <afxtempl.h>
#include "MpegTransfer.h"
#ifndef __MVRAPIDEF_H__
#include "MvrApiDef.h"
#endif

// for forward references
class CMpegSender;
class CMpegSenderThread;
class CMpegSenderWnd;

// private messages
#define MSG_ENC_STATUS		WM_APP
#define MSG_ENC_ERROR		(WM_APP + 1)

#define MPEG_PROGRAM_END_CODE	0xb9010000 // MPEG_program_end_code in reverse order

/////////////////////////////////////////////////////////////////////////////
// helpers

struct CEncCallback {
	UINT m_nEncId;
	CMpegSender* m_pSender;
	ENC_CB_STATUS m_pfnStatus;
	ENC_CB_ERROR m_pfnError;
	ENC_CB_VOBU_ENT m_pfnVobuEnt;

	CEncCallback(UINT nEncId, CMpegSender* pSender, ENC_CB_STATUS pfnStatus, ENC_CB_ERROR pfnError, ENC_CB_VOBU_ENT pfnVobuEnt);
	~CEncCallback();
};

inline CEncCallback::CEncCallback(UINT nEncId, CMpegSender* pSender,
	ENC_CB_STATUS pfnStatus, ENC_CB_ERROR pfnError, ENC_CB_VOBU_ENT pfnVobuEnt)
	: m_nEncId(nEncId), m_pSender(pSender), m_pfnStatus(pfnStatus), m_pfnError(pfnError), m_pfnVobuEnt(pfnVobuEnt)
	{}
inline CEncCallback::~CEncCallback()
	{}

/////////////////////////////////////////////////////////////////////////////
// CMpegSender

class CMpegSender : public CMpegTransfer
{
	DECLARE_DYNAMIC(CMpegSender)

// attributes
protected:
	static CCriticalSection m_csec; // for exclusive access to m_listEncCallbacks
	static CTypedPtrList<CPtrList, CEncCallback*> m_listEncCallbacks;
	static CCriticalSection m_csecThread; // for exclusive access to m_pSenderThread
	static CMpegSenderThread* m_pSenderThread;
	static CMpegSenderWnd* m_pSenderWnd;

	LPBYTE m_pbBssBuffAux; // auxiliary buffer

	friend CMpegSenderWnd;
	BOOL m_fStarting;
	BOOL m_fRestarting;

// operations
protected:
	static BOOL RegisterSender(UINT nEncId, CMpegSender* pSender);
	static void CALLBACK EncStatusCallback(UINT nEncId, ENC_STATUS_NOTIFY fStatus);
	static void CALLBACK EncErrorCallback(UINT nEncId, ULONG ulError);
	static void CALLBACK EncVobuCallback(UINT nEncId, ENC_VOBU_ENT_INFO* pInfo, UINT fBoundaryFlag);
	static void CALLBACK BssCallback(LPBYTE pbBuff, DWORD cbBuff, DWORD dwData);

public:
	CMpegSender(UINT nEncId);
	virtual ~CMpegSender();
	static BOOL Init(UINT nEncId, CMpegSender* pSender = NULL,
		ENC_CB_STATUS pfnStatus = NULL, ENC_CB_ERROR pfnError = NULL, ENC_CB_VOBU_ENT pfnVobuEnt = NULL);
	static BOOL Exit(UINT nEncId);
	static CEncCallback* FindEncCallback(UINT nEncId);

	BOOL Init(ENC_CB_STATUS pfnStatus = NULL, ENC_CB_ERROR pfnError = NULL, ENC_CB_VOBU_ENT pfnVobuEnt = NULL);
	BOOL Exit();
	BOOL SendStart(DWORD cbBssBuff = m_cbBuffDefault, DWORD cBssBuff = m_cBuffDefault);
	BOOL SendStop();
	int BssCallback(LPBYTE pbBuff, DWORD cbBuff);
};

inline BOOL CMpegSender::Init(ENC_CB_STATUS pfnStatus /* = NULL */, ENC_CB_ERROR pfnError /* = NULL */,
	ENC_CB_VOBU_ENT pfnVobuEnt /* = NULL */)
	{return Init(m_nId, this, pfnStatus, pfnError, pfnVobuEnt);}

inline BOOL CMpegSender::Exit()
	{return Exit(m_nId);}

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderThread

class CMpegSenderThread : public CWinThread
{
	DECLARE_DYNCREATE(CMpegSenderThread)
protected:
	CMpegSenderThread();			// IɎgpveNg RXgN^
									// protected constructor used by dynamic creation 

// Agr[g
// Attributes
public:

// Iy[V
// Operations
public:
	virtual ~CMpegSenderThread();

// I[o[Ch
// Overrides
	// ClassWizard ͉z֐̃I[o[Ch𐶐܂B
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMpegSenderThread)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Cve[V
// Implementation
protected:

	// ꂽbZ[W }bv֐
	// Generated message map functions
	//{{AFX_MSG(CMpegSenderThread)
		//  - ClassWizard ͂̈ʒuɃo֐ǉ܂͍폜܂B
		// NOTE: the ClassWizard will add member functions here
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CMpegSenderWnd

class CMpegSenderWnd : public CWnd
{
// RXgNV
// Construction
public:
	CMpegSenderWnd();

// Agr[g
// Attributes
public:
	BOOL m_bRestartReq;

// Iy[V
// Operations
public:

// I[o[Ch
// Overrides
	// ClassWizard ͉z֐̃I[o[Ch𐶐܂B
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMpegSenderWnd)
	protected:
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

// Cve[V
// Implementation
public:
	virtual ~CMpegSenderWnd();

	// ꂽbZ[W }bv֐
	// Generated message map functions
protected:
	//{{AFX_MSG(CMpegSenderWnd)
		//  - ClassWizard ͂̈ʒuɃo֐ǉ܂͍폜܂B
		// NOTE: the ClassWizard will add member functions here
	afx_msg void OnClose();
	afx_msg LRESULT OnEncStatus(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnEncError(WPARAM wParam, LPARAM lParam);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

