
// MpegTransfer.cpp : MPEG program stream transfer class implementations
// Copyright (C) 2000-2005 Canopus Co., Ltd. All rights reserved.

#include "stdafx.h"
#include "MpegTransfer.h"

/////////////////////////////////////////////////////////////////////////////
// CMpegTransfer

IMPLEMENT_DYNAMIC(CMpegTransfer, CObject)

WORD CMpegTransfer::m_wWsaVersion = 0; // windows socket version #, or 0 if WSAStartup fails

const DWORD CMpegTransfer::m_cbBuffDefault = 32768; // default BSS / BSR buffer size in bytes
const DWORD CMpegTransfer::m_cBuffDefault = 16; //default BSS / BSR buffer count

CMpegTransfer::CMpegTransfer(UINT nId /* = 1 */)
	: m_nId(nId), 
	m_fState(trsDisconnected), m_hSocket(INVALID_SOCKET), m_hEvent(WSA_INVALID_EVENT),
	m_nPort(0), m_cbBuff(m_cbBuffDefault), m_cBuff(m_cBuffDefault)
{
	memset(&m_Addr, 0, sizeof(SOCKADDR_IN));
	memset(&m_AddrOpposite, 0, sizeof(SOCKADDR_IN));
}

CMpegTransfer::~CMpegTransfer()
{
	Disconnect();
}

BOOL CMpegTransfer::StartUp()
{
	WSADATA WsaData;
	if (WSAStartup(MAKEWORD(2, 0), &WsaData) || LOBYTE(WsaData.wVersion) != 2 || HIBYTE(WsaData.wVersion) != 0)
		return FALSE;
	m_wWsaVersion = WsaData.wVersion;
	return TRUE;
}

BOOL CMpegTransfer::CleanUp()
{
	if (!m_wWsaVersion)
		return TRUE;
	else
	return (!WSACleanup()) ? TRUE : FALSE;
}

BOOL CMpegTransfer::SetSockAddrIn(LPCTSTR pszAddress, WORD nPort, LPSOCKADDR_IN pAddr)
{
	memset(pAddr, 0, sizeof(SOCKADDR_IN));
	LPSTR psz = T2A((LPSTR)pszAddress); // address must be a char, not a UNICODE
	pAddr->sin_family = AF_INET;
	pAddr->sin_port = htons(nPort);
	if ((pAddr->sin_addr.s_addr = inet_addr(psz)) == INADDR_NONE) {
		WSASetLastError(WSAEINVAL);
		return FALSE;
	}
	else
		return TRUE;
}

int CMpegTransfer::SendOverlapped(SOCKET hSocket, int fSocketType, LPVOID pBuff, DWORD cbBuff, LPDWORD pcbSent,
	LPSOCKADDR_IN pAddr /* = NULL */)
{
	if (fSocketType == SOCK_DGRAM)
		ASSERT(pAddr);
	*pcbSent = 0;
	int nResult = SOCKET_ERROR;
	WSAEVENT hEvent = WSA_INVALID_EVENT;
	if ((hEvent = WSACreateEvent()) == WSA_INVALID_EVENT)
		return nResult;
	WSABUF Buff;
	Buff.len = cbBuff; Buff.buf = (LPSTR)pBuff;
	WSAOVERLAPPED Overlapped;
	Overlapped.hEvent = hEvent;
	DWORD dwFlags = 0;
	if (!(nResult = (fSocketType == SOCK_STREAM) ?
		WSASend(hSocket, &Buff, 1, pcbSent, dwFlags, &Overlapped, NULL) :
		WSASendTo(hSocket, &Buff, 1, pcbSent, dwFlags, (LPSOCKADDR)pAddr, sizeof(SOCKADDR_IN), &Overlapped, NULL)))
		// operation is completed
		ASSERT(*pcbSent == cbBuff);
	else if (WSAGetLastError() == WSA_IO_PENDING && // operation is not completed
		WSAWaitForMultipleEvents(1, &hEvent, FALSE, INFINITE, FALSE) == WAIT_OBJECT_0 &&
		WSAGetOverlappedResult(hSocket, &Overlapped, pcbSent, FALSE, &dwFlags)) {
		nResult = 0;
		ASSERT(*pcbSent <= cbBuff && !dwFlags);
	}
	VERIFY(WSACloseEvent(hEvent));
	return nResult;
}

int CMpegTransfer::ReceiveOverlapped(SOCKET hSocket, int fSocketType, LPVOID pBuff, DWORD cbBuff, LPDWORD pcbReceived,
	LPSOCKADDR_IN pAddr /* = NULL */)
{
	if (fSocketType == SOCK_DGRAM)
		ASSERT(pAddr);
	*pcbReceived = 0;
	int nResult = SOCKET_ERROR;
	WSAEVENT hEvent = WSA_INVALID_EVENT;
	if ((hEvent = WSACreateEvent()) == WSA_INVALID_EVENT)
		return nResult;
	WSABUF Buff;
	Buff.len = cbBuff; Buff.buf = (LPSTR)pBuff;
	WSAOVERLAPPED Overlapped;
	Overlapped.hEvent = hEvent;
	DWORD dwFlags = 0;
	int cbAddr = sizeof(SOCKADDR_IN);
	if (!(nResult = (fSocketType == SOCK_STREAM) ?
		WSARecv(hSocket, &Buff, 1, pcbReceived, &dwFlags, &Overlapped, NULL) :
		WSARecvFrom(hSocket, &Buff, 1, pcbReceived, &dwFlags, (LPSOCKADDR)&pAddr, &cbAddr, &Overlapped, NULL)))
		// operation is completed
		ASSERT(*pcbReceived <= cbBuff);
	else if (WSAGetLastError() == WSA_IO_PENDING && // operation is not completed
		WSAWaitForMultipleEvents(1, &hEvent, FALSE, INFINITE, FALSE) == WSA_WAIT_EVENT_0 &&
		WSAGetOverlappedResult(hSocket, &Overlapped, pcbReceived, FALSE, &dwFlags)) {
		nResult = 0;
		ASSERT(*pcbReceived <= cbBuff && !dwFlags);
	}
	VERIFY(WSACloseEvent(hEvent));
	return nResult;
}

BOOL CMpegTransfer::Connect(LPCTSTR pszAddress, WORD nPort, LPCTSTR pszAddressOpposite)
{
	ASSERT(m_fState == trsDisconnected);
	m_nPort = nPort;
	if ((m_hEvent = WSACreateEvent()) == WSA_INVALID_EVENT)
		goto done;
	if (!SetSockAddrIn(pszAddress, m_nPort, &m_Addr) || !SetSockAddrIn(pszAddressOpposite, m_nPort, &m_AddrOpposite))
		goto done;
	// For client applications, the socket becomes bound implicitly through sendto, WSASendTo, or WSAJoinLeaf.
	// In other words, if a client application does not call sendto, etc, the socket never become bound, and,
	// WSARecvFrom failes with error code WSAEINVAL.
	// To workaround this, we explicitly bound the socket to local address.
	if ((m_hSocket = WSASocket(AF_INET, SOCK_DGRAM, 0, NULL, 0, WSA_FLAG_OVERLAPPED)) == INVALID_SOCKET ||
		bind(m_hSocket, (LPSOCKADDR)&m_Addr, sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
		goto done;
	m_fState = trsConnected;
done:
	if (m_fState == trsConnected)
		return TRUE;
	else {
		Disconnect();
		return FALSE;
	}
}

BOOL CMpegTransfer::Disconnect()
{
	if (m_hSocket != INVALID_SOCKET) {
		VERIFY(!closesocket(m_hSocket)); m_hSocket = INVALID_SOCKET;
	}
	if (m_hEvent != WSA_INVALID_EVENT) {
		VERIFY(WSACloseEvent(m_hEvent)); m_hEvent = WSA_INVALID_EVENT;
	}
	m_fState = trsDisconnected;
	return TRUE;
}

